/* Nilo - A Network boot loader.

   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* Define the PXE types */
#include "pxe_types.h"

/* Define the PXE data structures */
#include "pxe.h"

/* Define the PXE UDP data structures */
#include "pxe_udp.h"

/* Define the tracing macros */
#include "trace.h"

/* Define the data structure dumping functions for debug use */
#include "dump.h"

#include "nilo.h"

/* Define the standard BSD TFTP data types */
#include <oskit/freebsd/sys/types.h>
#include <net/if.h>
#include <arpa/tftp.h>
#include <net/if_arp.h>

#include <stdlib.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include "tagged.h"
#include "util.h"

#include "pxenv.h"
#include "pxe.h"
#include "pxe_undi.h"
#include "pxe_tftp.h"

#ifdef NILO_USE_MULTIBOOT
extern oskit_addr_t reboot_stub_loc;
#include <oskit/x86/pc/base_multiboot.h> /* boot_info */
#include <assert.h>
#else
#endif /* NILO_USE_MULTIBOOT */
int oskit_bootargc;
char **oskit_bootargv;
#undef	SIMULATE_BLOCK_LOSS

#include <oskit/gdb.h>

extern u_int32_t jiffies;

struct client_data *client_info;
static int32_t timeout;
static u_int32_t xid;

pxe_t *pxe_data_struct;
pxenv_t *pxenv_data_struct;

int32_t seed;	/* for random number generator */

// static void dhcp_callback (int32_t snaplen, dhcp_t *dhcp_ptr);
// static void process_dhcp_reply (dhcp_t *dhcp_ptr);

/**************************************************************************
RFC951_SLEEP - sleep for expotentially longer times
**************************************************************************/
int32_t
rfc951_sleep(int32_t exp)
{
    int32_t q,z,tmo;

    NILO_TRACE_FUNCTION;

    /* simplified version of the LCG given in Bruce Scheier's
       "Applied Cryptography" */
    q = seed/53668;
    if ((seed = 40014*(seed-53668*q) - 12211*q) < 0) seed += 2147483563l;
    /* compute mask */
    for (tmo = 63; tmo <= 60*1008 && --exp > 0; tmo = 2*tmo+1);
    return (tmo & seed);

    NILO_TRACE_FUNCTION_RETURN;
}

static void
jump_to_code(void)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
}

#ifdef NILO_USE_MULTIBOOT
/* We grabbed these from OSKit's netboot, cause these process the
   console options so we can use the serial port */

/*
 * Figure out if -h (serial) or -f (fast serial) are in the oskit_bootargv
 * and put them in the default_args list.
 */
static void
init_default_args(int bootargc, char *bootargv[], char *default_args[])
{
    char **p;
    int i;
    
    NILO_TRACE_FUNCTION;

    p = &default_args[0];
    for (i = 0; i < bootargc; i++)
#define NDEFAULT_ARGS 2
        if (strcmp(bootargv[i], "-h") == 0 ||
            strcmp(bootargv[i], "-f") == 0)
            *p++ = bootargv[i];
    *p = NULL;

    NILO_TRACE_FUNCTION_RETURN;
}

/*
 * Build a command line from the program name, the default_args,
 * what they specified, and our -retaddr option.
 *
 * The command line is in a special format understood by the oskit
 * multiboot startup code (see base_multiboot_init_cmdline.c).
 * Basically, the format separates booting-options and environment
 * variable settings from normal args to pass to main().  The format is like
 *      progname [<booting-options and foo=bar> --] <args to main>
 * For example
 *      kernel DISPLAY=host:0 -d -- -rf foo bar
 */
static char *
build_cmdline(char *progname, char *default_args[NDEFAULT_ARGS], char *input)
{
    char *toks[input? strlen(input) : 0];
    char *tok;
    int ntoks;
    int i;
    int havesep;
    int cllen;
    char *cmdline;
    NILO_TRACE_FUNCTION;

    /* Tokenize their input. */
    ntoks = 0;
    if (input)
        for (tok = strtok(input, " \t"); tok; tok = strtok(0, " \t"))
            toks[ntoks++] = tok;
    
    /* Figure out if they specified a "--". */
    havesep = 0;
    for (i = 0; i < ntoks; i++)
        if (strcmp(toks[i], "--") == 0) {
            havesep = 1;
            break;
        }
    
    /*
     * Toggle off any bootopts that they specified.
     * We only have to do this if there is a --, since otherwise
     * they didn't specify any bootopts.
     */
    if (havesep) {
        for (i = 0; i < ntoks; i++) {
            int j;
            
            if (strcmp(toks[i], "--") == 0)
                break;
            for (j = 0; j < NDEFAULT_ARGS && default_args[j]; j++)
                if (strcmp(toks[i], default_args[j]) == 0) {
                    toks[i] = 0;
                    default_args[j] = 0;
                }
        }
    }
    
    /* Figure out how big the command-line will be and allocate it. */
    cllen = strlen(progname) + 1;
    cllen += strlen("-retaddr 0x00ff00ff") + 1;
    for (i = 0; i < NDEFAULT_ARGS; i++)
        if (default_args[i])
            cllen += strlen(default_args[i]) + 1;
    if (! havesep)
        cllen += strlen("--") + 1;
    for (i = 0; i < ntoks; i++)
        if (toks[i])
            cllen += strlen(toks[i]) + 1;
    cllen += 1;                     /* for NUL */
    
    cmdline = mustcalloc(cllen,1);
    assert(cmdline);
    
    /*
     * Now fill it in.
     */
    strcpy(cmdline, progname);
    strcat(cmdline, " ");
    
//    sprintf(cmdline + strlen(progname) + 1, "-retaddr 0x%08x",
//            reboot_stub_loc);
//    strcat(cmdline, " ");
    
    for (i = 0; i < NDEFAULT_ARGS; i++)
        if (default_args[i]) {
            strcat(cmdline, default_args[i]);
            strcat(cmdline, " ");
        }
    
    if (! havesep) {
        strcat(cmdline, "--");
        strcat(cmdline, " ");
    }
    
    for (i = 0; i < ntoks; i++)
        if (toks[i]) {
            strcat(cmdline, toks[i]);
            strcat(cmdline, " ");
        }
    
    NILO_TRACE_FUNCTION_RETURN;
    return cmdline;
}
#endif /* NILO_USE_MULTIBOOT */

int 
main(int argc, char **argv)
{
    int32_t retries;
    struct in_addr addr;
    pxenv_undi_initialize_t undiinit;
    pxenv_udp_open_t udpopen;
    pxenv_tftp_read_file_t tftpfile;

#ifdef NILO_USE_MULTIBOOT
    char *default_args[NDEFAULT_ARGS+1] = {0};
#endif /* NILO_USE_MULTIBOOT */

    NILO_TRACE_FUNCTION;
    
    NILO_TRACE_PRINTF ("NILO dated " VERSION "\r\n");
    init_system();

#ifdef NILO_USE_MULTIBOOT
    init_default_args(oskit_bootargc, oskit_bootargv, default_args);
#endif /* NILO_USE_MULTIBOOT */

    if ((client_info = malloc(sizeof(*client_info))) == 0) {
        NILO_TRACE_MSG("Malloc failed\r\n");
    }

    if (CALL_PXE_API (PXENV_UNDI_INITIALIZE, &undiinit) == PXENV_EXIT_FAILURE) {
        panic ("Couldn't initialize the UNDI system\r\n");
        return (PXENV_EXIT_FAILURE);
    }

    /* Open the UDP connection. This also does a DHCP session to get
       the boot parameters. */
    if (CALL_PXE_API (PXENV_UDP_OPEN, &udpopen) == PXENV_EXIT_FAILURE) {
        panic ("Couldn't open the UDP connection.\r\n");
        return (PXENV_EXIT_FAILURE);
    }

    /* The IP address of the TFTP server in Big Endian format. */
    tftpfile.server_ip.num = client_info->siaddr.num;
    /* The IP address of the TFTP gateway in Big Endian format. */
    tftpfile.gw_ip.num = client_info->yiaddr.num;
    /* The size of the receive buffer in bytes. */
    tftpfile.buffersize = TFTP_DEFAULTSIZE_PACKET;
    /* The timeout value in seconds used for receiving data. If zero,
       the default of TFTP timeout is used. */
    tftpfile.tftp_open_timeout = 0;
    /* The maximum wait delay time in seconds between ACK of a fast
       packet and a new TFTP open. */
    tftpfile.tftp_reopen_delay = 10;
    /* The bootfile name. */
    strcpy (tftpfile.filename, client_info->file);
    /* the port the client listens to for a multicast. */
    tftpfile.tftp_clnt_port = TFTP_CLIENT_PORT;
    client_info->tftp_client_port = TFTP_CLIENT_PORT;
    /* The port the server sends to for a multicast. */
    tftpfile.tftp_srv_port = TFTP_SERVER_PORT;

    /* Load a file via a (closed) TFTP connection */
    if (CALL_PXE_API (PXENV_TFTP_READ_FILE, &tftpfile) == PXENV_EXIT_FAILURE) {
        panic ("Couldn't open the TFTP connection, and load a file.\r\n");
        return (PXENV_EXIT_FAILURE);
    }

    /* Start the TFTP transfer */
    //    client_info->tftp_handler(client_info->siaddr.num, TFTP_RRQ,
    //                      client_info->file, client_info->tftp_blockno);

    /* execute the new image */
    jump_to_code();

    NILO_TRACE_FUNCTION_RETURN;

    return (0);
}






