/* 
   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* Define the PXE types */
#include "pxe_types.h"

/* Define the tracing macros */
#include "trace.h"

/* Define the TFTP data structures */
#include "pxe_undi.h"

#include "nilo.h"

#include <oskit/dev/dev.h>
#include <oskit/dev/net.h>
#include <oskit/dev/error.h>
#include <oskit/net/ether.h>
#include <oskit/dev/ethernet.h>
#include <oskit/dev/linux.h>
#include <oskit/dev/device.h>
#include <oskit/io/netio.h>
#include <oskit/com/queue.h>
#include <assert.h>

#define PACKQ_MAX 32
extern oskit_queue_t *packq;

extern struct client_data *client_info;

void bumpticks(void);
static struct etherdev devs[MAX_DEVICES];

extern oskit_error_t polling_net_receive(void *data, oskit_bufio_t *b, oskit_size_t pkt_size);
extern oskit_error_t net_receive(void *data, oskit_bufio_t *b, oskit_size_t pkt_size);

unsigned char bcastaddr[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};

/* pxenv_undi_startup() - initialize the UNDI code & data segments. This
 *      must be called before any of the other UNDI functions.
 *
 * parameters:
 *     none
 * returns:
 *     undistartup.status - the exit status of this function. This is set
 *                          to one of the PXENV_STATUS_xxx constants.
 */
pxenv_ret_code_t
pxenv_undi_startup (pxenv_undi_startup_t *undistartup)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_cleanup() - prepares the driver to be unloaded from memory.
 *      After this is executed, the UNDI functions won't work.
 *      Note: This cannot be used in protected mode.
 *
 * parameters:
 *     none
 * returns:
 *     undicleanup.status - the exit status of this function. This is set
 *                          to one of the PXENV_STATUS_xxx constants.
 */
pxenv_ret_code_t
pxenv_undi_cleanup (pxenv_undi_cleanup_t *undicleanup)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_initialize() - initializes the adaptor with the data as set
 *      by the most recent pxenv_undi_startup(). A pxenv_undi_open() must
 *      be executed before the adaptor can transmit packets.
 *
 * parameters:
 *     undiinit.protocolini - this is the 32 bit address of the memory
 *                   copy of the driver module in the protocol.ini file. \
 *                   This used to pass into to the ndis driver. Since we're #                   not on NT, we don't care.
 *     undiinit.reserved - FIXME: what is this ?
 * returns:
 *     undiinit.status - the exit status of this function. This is set
 *                       to one of the PXENV_STATUS_xxx constants.

 */
pxenv_ret_code_t
pxenv_undi_initialize (pxenv_undi_initialize_t *undiinit)
{
    char *p;
    oskit_error_t err;
    oskit_etherdev_t **etherdev;
    int i, a0, a1, a2, a3, a4, a5;
    
    NILO_TRACE_FUNCTION;

    NILO_TRACE_MSG ("Initializing devices...\r\n");


    /* The cast is needed to make it non-volatile. */
    err = oskit_bounded_com_queue_create(PACKQ_MAX, (oskit_queue_t **)&packq);
    if (err)
      return err;
    
    oskit_dev_init();
    oskit_linux_init_net();
    oskit_dump_drivers();
    
    NILO_TRACE_MSG("Probing devices...\n");
    oskit_dev_probe();
    oskit_dump_devices();

    osenv_timer_register(bumpticks, DHCP_TIMER_FREQ);

    /* Find all the Ethernet device nodes. */
   NILO_TRACE_MSG("Looking for devices...\r\n");
     client_info->ndev = osenv_device_lookup(&oskit_etherdev_iid, (void***)&etherdev);
    if (client_info->ndev <= 0)
        panic("no Ethernet adaptors found!");
    if (client_info->ndev > MAX_DEVICES)
        panic("XXX too many Ethernet adaptors found");
    
    /* Open all the devices and have them hand packets to the OS. */
    NILO_TRACE_MSG("%d Ethernet adaptor%s found:\r\n", client_info->ndev,
                   client_info->ndev > 1 ? "s" : "");
    for (i = 0; i < client_info->ndev; i++) {
        int j;
        
        err = oskit_etherdev_getinfo(etherdev[i], &devs[i].info);
        if (err)
            panic("error getting info from ethercard %d", i);
        
        oskit_etherdev_getaddr(etherdev[i], devs[i].haddr);
        
        /* Show information about this adaptor */
        NILO_TRACE_PRINTF("  %-16s%-40s ", devs[i].info.name,
               devs[i].info.description
               ? devs[i].info.description : "");
        for (j = 0; j < 5; j++) 
            NILO_TRACE_PRINTF ("%02x:", devs[i].haddr[j]);
        NILO_TRACE_PRINTF("%02x\r\n", devs[i].haddr[5]);
        
        devs[i].dev = etherdev[i];
        devs[i].recv_nio = oskit_netio_create(polling_net_receive, &devs[i]);
        if (devs[i].recv_nio == NULL)
            panic("unable to create recv_nio");
        
        err = oskit_etherdev_open(etherdev[i], 0, devs[i].recv_nio,
                                  &devs[i].send_nio);
    }
    
    if (!osenv_intr_enabled()) {
        NILO_TRACE_MSG("Interrupts not enabled! enabling!\r\n");	
        osenv_intr_enable();
    }
    
    /* Store the device info in the global data structure. We have to pick
       one, so for now we have a FIXME: default to the first and probably
       only one */
    memcpy ((int8_t *)&client_info->nic, (int8_t *)&devs[0], sizeof(struct etherdev));
    client_info->macaddr[0] = client_info->nic.haddr[0];
    client_info->macaddr[1] = client_info->nic.haddr[1];
    client_info->macaddr[2] = client_info->nic.haddr[2];
    client_info->macaddr[3] = client_info->nic.haddr[3];
    client_info->macaddr[4] = client_info->nic.haddr[4];
    client_info->macaddr[5] = client_info->nic.haddr[5];

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_reset_nic() - resets the adaptor to the default values
 *      as set with pxenv_undi_initialize() function. Unlike the
 *      pxenv_undi_initialize() function, this connects the adaptor to the
 *      network. 
 *
 * parameters:
 *     protocolini - this is the 32 bit address of the memory copy of
 *                   the driver module in the protocol.ini file. This
 *                   used to pass into to the ndis driver.
 *     reserved - FIXME: what is this ?
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
pxenv_ret_code_t
pxenv_undi_reset_nic (pxenv_undi_reset_nic_t *undireset)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_shutdown() - resets the adaptor and leaves it in a usable
 *      state. 
 *      Note: The data passed to the pxenv_undi_startup() function in the
 *      pxenv_undi_startup_s structure must be saved for the next call to
 *      pxenv_undi_initialize(). 
 *
 * parameters:
 *     none
 * returns:
 *     undishut.status - the exit status of this function. This is set
 *                       to one of the PXENV_STATUS_xxx constants.
 */
pxenv_ret_code_t
pxenv_undi_shutdown (pxenv_undi_shutdown_t *undishut)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_open() - activates the adaptor so it can accept packets.
 * 
 * parameters:
 *     undiopen.open_flag - this is an adaptor specific flag used to pass the 
 *                 Open flags from the protocol driver. Can be zero.
 *     undiopen.pkt_filter - one of the constants below (can be ORed together).
 * returns:
 *     undiopen.status - the exit status of this function. This is set
 *                       to one of the PXENV_STATUS_xxx constants.
 */
pxenv_ret_code_t
pxenv_undi_open (pxenv_undi_open_t *undiopen)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_close() - disconnects the adaptor from the network. The
 *      adaptor must be opened with a call to pxenv_undi_open() before
 *      it ca be used again. 
 *
 * parameters:
 *     none
 * returns:
 *     undiclose.status - the exit status of this function. This is set
 *                        to one of the PXENV_STATUS_xxx constants.
 */
pxenv_ret_code_t
pxenv_undi_close (pxenv_undi_close_t *undiclose)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_transmit() - transmits a packet over the network. An
 *      interupt is generated on the transmissions of a packet(s).
 *
 * parameters:
 *     unditrans.protocol - the protocol of the upper layer that is making
 *                this function call. If the upper layer has filled in 
 *                the media header, this must be zero.
 *     unditrans.xmit_flags - if zero, use the dest_addr. If 1, the adapator
 *                 supplies the broadcast address.
 *     unditrans.dest_addr - the hardware address of the destination media.
 *                 If this isn't known, this field can be zero, and
 *                 xmit_flags must be XMT_BROADCAST.
 *     unditrans.buffer - the transmit.
 * returns:
 *     unditrans.status - the exit status of this function. This is set
 *                        to one of the PXENV_STATUS_xxx constants.
 */
pxenv_ret_code_t
pxenv_undi_transmit (pxenv_undi_transmit_t *unditrans)
{
    oskit_bufio_t *b;
    u_int16_t type;
    oskit_error_t err = OSKIT_E_DEV_IOERR;
    int offset = 0;
    oskit_size_t actual;
        
    
    NILO_TRACE_FUNCTION;

    /* allocate a buffer to send */
//    b = oskit_bufio_create(6+6+sizeof(unditrans->protocol) +
    b = oskit_bufio_create(6+6+2+ unditrans->tbd_addr->length);
    if (b != NULL) {
        /* fill in the ethernet packet */
      /* fill in the dest addr if xmit flags is zero */
      // if (unditransmit.xmit_flags == XMT_BROADCAST)
      // memcpy (unditrans.dest_addr, bcastaddr, sizeof(bcastaddr));
        oskit_bufio_write(b, &unditrans->dest_addr, offset, 6, &actual);
        assert(actual == 6);
        offset += 6;
        
        /* fill in source hardware address */
        oskit_bufio_write(b, client_info->macaddr, offset, 6, &actual);
        assert(actual == 6);
        offset += 6;
        
        switch (unditrans->protocol) {
        case P_IP:
            type = htons(ETHERTYPE_IP);
            break;
        case P_ARP:
            type = htons(ETHERTYPE_ARP);
            break;
        default:
            type = 0;
            break;
        }
        
        oskit_bufio_write(b, &type, offset,
                          sizeof(type), &actual);
        assert(actual == sizeof(type));
        offset += sizeof(type);
        
        oskit_bufio_write(b, (void *)unditrans->tbd_addr->xmit_buffer,
                          offset, unditrans->tbd_addr->length, &actual);
        assert(actual == unditrans->tbd_addr->length);
        offset += unditrans->tbd_addr->length;

        /* okay, `push' the packet onto the wire */
        err = oskit_netio_push(client_info->nic.send_nio, b,
                               6+6+sizeof(type) + 
                               unditrans->tbd_addr->length);
        
        oskit_bufio_release(b);
    }
    
//    unditrans.status = 

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_mcast_addr() - changes the list of muticast addresses to
 *      the specified list, and resets the adaptor. If there are no
 *      multicast addresses, then multicast is disabled.
 *
 * parameters:
 *     undimcast.mcast_buffer - the multicast address.
 * returns:
 *     undimcast.status - the exit status of this function. This is set
 *                        to one of the PXENV_STATUS_xxx constants.
 */
pxenv_ret_code_t
pxenv_undi_set_mcast_addr (pxenv_undi_set_mcast_addr_t *undimcast)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_set_station_addr() - sets the MAC address to the specified
 *      value, and must be executed before opening the network adaptor
 *      with the pxenv_undi_open() function.
 *
 * parameters:
 *     undistat.station_addr - the MAC address to be used.
 * returns:
 *     undistat.status - the exit status of this function. This is set
 *                       to one of the PXENV_STATUS_xxx constants.
 */
pxenv_ret_code_t
pxenv_undi_set_station_addr (pxenv_undi_set_station_addr_t *undistat)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_set_packet_filter() - resets the adaptor to use a new filter.
 *
 * parameters:
 *     undifilter.filter - the new filter for the NIC receive unit.
 *                         Permissable values are FLTR_DIRECTED,
 *                         FLTR_BRDCST, FLTR_PRMSCS, FLTR_SRC_RTG.
 * returns:
 *     undifilter.status - the exit status of this function. This is set
 *                         to one of the PXENV_STATUS_xxx constants.
 */
pxenv_ret_code_t
pxenv_undi_set_packet_filter (pxenv_undi_set_packet_filter_t *undifilter)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_get_information() - copies all the adaptor variables to
 *      the input buffer.
 *
 * parameters:
 *     none
 * returns:
 *     undiinfo.status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 *     undiinfo.baseio - the base I/O address of the adpator.
 *     undiinfo.irqnum - the IRQ number of the adaptor.
 *     undiinfo.maxmtu - the maxium MTUs. FIXME:
 *     undiinfo.hwtype - the protocol type at the hardware level. There is an
 *              RFRC1010 for "Assigned Numbers". As this number may
 *              not be validated by the application and hence adding
 *              new numbers to the list should be fine at any time.
 *     undiinfo.hw_addrlen - the length of the hardware address.
 *     undiinfo.current_node_addr - the current hardware address.
 *     undiinfo.perm_node_addr - the permanent hardware address.
 *     undiinfo.rom_addr - the ROM address.
 *     undiinfo.rxbuflen - the length of the receive buffer in bytes.
 *     undiinfo.txbuflen - the length of the transmit buffer in bytes.
 */
pxenv_ret_code_t
pxenv_undi_get_information (pxenv_undi_get_information_t *undiinfo)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_get_statistics() - gets the adaptor statistics.
 *
 * parameters:
 *     none
 * returns:
 *     undistats.status - the exit status of this function. This is set
 *                        to one of the PXENV_STATUS_xxx constants.
 *     undistats.tx_good_frames - the count of good transmissions.
 *     undistats.rx_good_frames - the count of good receptions.
 *     undistats.rx_crc_err - the count of frames with CRC errors.
 *     undistats.rx_discard_err - the count of rames discarded.
 */
pxenv_ret_code_t
pxenv_undi_get_statistics (pxenv_undi_get_statistics_t *undistats)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_clear_statistics() - clears the statistics from the adaptor.
 *
 * parameters:
 *     none
 * returns:
 *     undiclrstats.status - the exit status of this function. This is set
 *                           to one of the PXENV_STATUS_xxx constants.
 */
pxenv_ret_code_t
pxenv_undi_clear_statistics (pxenv_undi_clear_statistics_t *undiclrstats)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_initiate_diags() - starts the run-time diagnostics, typically
 *      by running the hardware diagnostics.
 *
 * parameters:
 *     none
 * returns:
 *     undidiag.status - the exit status of this function. This is set
 *                       to one of the PXENV_STATUS_xxx constants.
 */
pxenv_ret_code_t
pxenv_undi_initiate_diags (pxenv_undi_initiate_diags_t *undidiag)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_force_interupts() - forces the adaptor to generate and
 *      interupt. This is done to simulate and interupt on receiving
 *      a packet to clear the adaptor.
 *
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
pxenv_ret_code_t
pxenv_undi_force_interrupt (pxenv_undi_force_interrupt_t *undiint)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_get_mcast_addr() - converts the specified IP multicast
 *      address to it's hardware multicast address.
 *
 * parameters:
 *     undimcast.ip_addr - the multicast IP number.
 * returns:
 *     undimcast.status - the exit status of this function. This is set
 *                        to one of the PXENV_STATUS_xxx constants.
 *     undimcast.mcast_addr - the hardware multicast address.
 */
pxenv_ret_code_t
pxenv_undi_get_mcast_addr (pxenv_undi_get_mcast_addr_t *undimcast)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_get_nic_type() - retrieves the adaptor specifc info required
 *      to identify the adaptor.
 *      Note: pxenv_start_undi(), pxenv_undi_startup(), and
 *      pxenv_undi_initialize() must be called before this info is valid.
 *
 * parameters:
 *     none
 * returns:
 *     undinic.status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 *     undinic.nictype - This is set to 0x2 if a PCI card, or 0x3 if PNP
 *   There is one of two data structures filled in, depending wether
 *   it's a PCI or a PNP adaptor card.
 *     undinic.pci.vendor_id - the vendor ID.
 *     undinic.pci.dev_id - the device ID.
 *     undinic.pci.base_class - the base class of the device.
 *     undinic.pci.sub_class - the sub class of the device.
 *     undinic.pci.prog_inf - the program interface.
 *     undinic.pci.rev - the revision number.
 *     undinic.pci.busdevfunc - bus device and function numbers.
 *     undinic.pci.subvendor_id - the sub vendor ID.
 *     undinic.pci.subdevice_id - the sub device ID.
 *   or
 *     undinic.pnp.eisa_dev_id - the EISA device ID.
 *     undinic.pnp.base_class - the base class of the device.
 *     undinic.pnp.sub_class - the sub class of the device.
 *     undinic.pnp.prog_intf - the program interface.
 *     undinic.pnp.cardselnum - the card selector number 
 *     undinic.pnp.reserved - data structure padding to make this aligned.
 */
pxenv_ret_code_t
pxenv_undi_get_nic_type (pxenv_undi_get_nic_type_t *undinic)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_get_iface_info() - gets the link level infterface type and
 *      the link speed.
 *      Note: pxenv_start_undi(), pxenv_undi_startup(), and
 *      pxenv_undi_initialize() must be called before this info is valid.
 *      Also the info must be in an NDIS2 format.
 *
 * parameters:
 *     none
 * returns:
 *     undiiface.status - the exit status of this function. This is set
 *                        to one of the PXENV_STATUS_xxx constants.
 *     undiiface.iface_type - this is the type of interface in ASCIIZ format.
 *     undiiface.link_speed - as defined in the NDIS 2.0x specification.
 *     undiiface.service_flags - as defined in the NDIS 2.0x specification.
 *     undiiface.reserved - must be set to all zeros as it's undefined for now.
 */
pxenv_ret_code_t
pxenv_undi_get_iface_info (pxenv_undi_get_iface_info_t *undiiface)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_get_state() - gets the state of the UNDI system.
 *
 * parameters:
 *     none
 * returns:
 *     undistate.status - the exit status of this function. This is set
 *                        to one of the PXENV_STATUS_xxx constants.
 *     undistate.undi_state - the state of the UNDI engine.
 */
pxenv_ret_code_t
pxenv_undi_get_state (pxenv_undi_get_state_t *undistate)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}

/* pxenv_undi_isr() - this is the Interupt Service Routine for the UNDI.
 *
 * parameters:
 *     undiisr.func_flag - this is set to one of the PXENV_UNDI_ISR_IN_*
 *              constants.
 * returns:
 *     undiisr.status - the exit status of this function. This is set
 *                      to one of the PXENV_STATUS_xxx constants.
 *     undiisr.func_flag - this is set to one of the PXENV_UNDI_ISR_OUT_*
 *              constants.
 *     undiisr.buffer_length - the length of the buffer in bytes.
 *     undiisr.frame_length - the length of the frame in bytes.
 *     undiisr.frame_header_length - the length of the frame header in bytes.
 *     undiisr.frame_addr - the frame address in offset:segment format.
 *     undiisr.prot_type - the type of the protocol. Valid values are:
 *                 1 - PXENV_UNDI_PROTO_IP
 *                 2 - PXENV_UNDI_PROTO_ARP
 *                 3 - PXENV_UNDI_PROTO_RARP
 *                 4 - other
 *     undiisr.pkt_type - the type of packet. Valid values are:
 *                 0 - directed/promiscuous
 *                 1 - broadcast
 *                 2 - multicast
 */
pxenv_ret_code_t
pxenv_undi_isr (pxenv_undi_isr_t *undiisr)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
    return (PXENV_EXIT_SUCCESS);
}
