/* Nilo - A Network boot loader.

   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* Define the PXE types */
#include "pxe_types.h"

/* Define the PXE data structures */
#include "pxe.h"

/* Define the PXE UDP data structures */
#include "pxe_udp.h"

/* Define the PXE UDP data structures */
#include "pxe_tftp.h"

/* Define the PXE UNDI data structures */
#include "pxe_undi.h"

/* Define the PXE preboot data structures */
#include "pxe_preboot.h"

#include <oskit/dev/error.h>

/* Define the tracing macros */
#include "trace.h"

/* Define the data structure dumping functions for debug use */
#include "dump.h"

#include "arp.h"
#include "dump.h"
#include "nilo.h"

#include "dejagnu.h"

/* Define the standard BSD TFTP data types */
#include <oskit/freebsd/sys/types.h>

#include <stdlib.h>
#include "util.h"

#include "pxe.h"
#include "pxenv.h"

void test_call_api(void);

static int called_func;
static pxe_t *pxe_data_struct;
static pxenv_t *pxenv_data_struct;

oskit_error_t net_receive(void *data, oskit_bufio_t *b, oskit_size_t pkt_size);

/* We use this macro to test for the specified vector in the
   table of functions supported by the PXE API. */
#define apitest(x, msg) \
        called_func = 0; \
        pxe_data_struct->entry32point (x, (void *)0); \
        (called_func != x) ? \
        fail (msg " not in PXE API table") : \
        pass (msg " is in the PXE API table")

int 
main(int argc, char **argv)
{
//    NILO_TRACE_FUNCTION;
        
    pxenv_data_struct = find_pxe_data(PXENV);
    pxe_data_struct = find_pxe_data(PXE);

    /* The PXENV data structure should have a pointer to the PXE
       data structure, so if we've found both, they should match. */
    if (pxenv_data_struct->runtimeptr == pxe_data_struct) {
        pass ("Found both the PXE and PXENV data structures");
    } else {
        fail ("Didn't find both the PXE and PXENV data structures");
        exit (-1);
    }

    test_call_api();

    totals();

//    NILO_TRACE_FUNCTION_RETURN;
}

void
test_call_api(void)
{
//    NILO_TRACE_FUNCTION;
    /* test the UDP functions */
    apitest (PXENV_UDP_OPEN,  "pxenv_udp_open()");
    apitest (PXENV_UDP_READ,  "pxenv_udp_read()");
    apitest (PXENV_UDP_WRITE, "pxenv_udp_write()");
    apitest (PXENV_UDP_CLOSE, "pxenv_udp_close()");

    /* test the TFTP functions */
    apitest (PXENV_TFTP_OPEN,  "pxenv_tftp_open()");
    apitest (PXENV_TFTP_READ,  "pxenv_tftp_read()");
    apitest (PXENV_TFTP_READ_FILE, "pxenv_udp_read_file()");
    apitest (PXENV_TFTP_GET_FSIZE, "pxenv_tftp_get_fsize()");

    /* test the PREBOOT PXE API functions */
    apitest (PXENV_UNLOAD_STACK, "pxenv_unload_stack()");
    apitest (PXENV_GET_BINL_INFO, "pxenv_get_cached_info()");
    apitest (PXENV_RESTART_TFTP, "pxenv_restart_tftp()");
    apitest (PXENV_START_UNDI,   "pxenv_start_undi()");
    apitest (PXENV_STOP_UNDI,    "pxenv_stop_undi()");
    apitest (PXENV_START_BASE,   "pxenv_start_base()");
    apitest (PXENV_STOP_BASE,    "pxenv_stop_base()");

    /* test the UNDI PXE API functions*/
    apitest (PXENV_UNDI_CLEANUP, "pxenv_undi_cleanup()");
    apitest (PXENV_UNDI_INITIALIZE, "pxenv_undi_initialize()");
    apitest (PXENV_UNDI_RESET_NIC, "pxenv_undi_reset_nic()");
    apitest (PXENV_UNDI_SHUTDOWN, "pxenv_undi_shutdown()");
    apitest (PXENV_UNDI_OPEN, "pxenv_undi_open()");
    apitest (PXENV_UNDI_CLOSE, "pxenv_undi_close()");
    apitest (PXENV_UNDI_TRANSMIT, "pxenv_undi_transmit()");
    apitest (PXENV_UNDI_SET_MCAST_ADDR, "pxenv_undi_set_mcast_addr()");
    apitest (PXENV_UNDI_SET_STATION_ADDR, "pxenv_undi_set_station_addr()");
    apitest (PXENV_UNDI_SET_PACKET_FILTER, "pxenv_undi_set_packet_filter()");
    apitest (PXENV_UNDI_GET_INFORMATION, "pxenv_undi_get_information()");
    apitest (PXENV_UNDI_GET_STATISTICS, "pxenv_undi_get_statistics()");
    apitest (PXENV_UNDI_CLEAR_STATISTICS, "pxenv_undi_clear_statistics()");
    apitest (PXENV_UNDI_INITIATE_DIAGS, "pxenv_undi_initiate_diags()");
    apitest (PXENV_UNDI_FORCE_INTERRUPT, "pxenv_undi_force_interrupt()");
    apitest (PXENV_UNDI_GET_MCAST_ADDR, "pxenv_undi_get_mcast_addr()");
    apitest (PXENV_UNDI_GET_NIC_TYPE, "pxenv_undi_get_nic_type()");
    apitest (PXENV_UNDI_ISR, "pxenv_undi_isr()");

//    NILO_TRACE_FUNCTION_RETURN;
}

oskit_error_t
net_receive (void *data, oskit_bufio_t *b, oskit_size_t pkt_size)
{
}

/* UDP functions stubs */
pxenv_ret_code_t
pxenv_udp_open (pxenv_udp_open_t *udpopen)
{
    called_func = PXENV_UDP_OPEN;
}

pxenv_ret_code_t
pxenv_udp_close (pxenv_udp_close_t *udpclose)
{
    called_func = PXENV_UDP_CLOSE;
}

pxenv_ret_code_t
pxenv_udp_read (pxenv_udp_read_t *udpread)
{
    called_func = PXENV_UDP_READ;
}

pxenv_ret_code_t
pxenv_udp_write (pxenv_udp_write_t *udpwrite)
{
    called_func = PXENV_UDP_WRITE;
}

/* stubs for the PXE TFTP API functions */
pxenv_ret_code_t
pxenv_tftp_open (pxenv_tftp_open_t *tftpopen)
{
    called_func = PXENV_TFTP_OPEN;
}

pxenv_ret_code_t
pxenv_tftp_close (pxenv_tftp_close_t *tftpclose)
{
    called_func = PXENV_TFTP_CLOSE;
}

pxenv_ret_code_t
pxenv_tftp_read (pxenv_tftp_read_t *tftpread)
{
    called_func = PXENV_TFTP_READ;
}

pxenv_ret_code_t
pxenv_tftp_read_file (pxenv_tftp_read_file_t *tftpfile)
{
    called_func = PXENV_TFTP_READ_FILE;
}

pxenv_ret_code_t
pxenv_tftp_get_fsize (pxenv_tftp_get_fsize_t *tftpfsize)
{
    called_func = PXENV_TFTP_GET_FSIZE;
}

/* stubs for the PXE UNDI API functions */
pxenv_ret_code_t
pxenv_undi_startup (pxenv_undi_startup_t *undistartup)
{
    called_func = PXENV_UNDI_STARTUP;
}

pxenv_ret_code_t
pxenv_undi_cleanup (pxenv_undi_cleanup_t *undicleanup)
{
    called_func = PXENV_UNDI_CLEANUP;
}

pxenv_ret_code_t
pxenv_undi_initialize (pxenv_undi_initialize_t *undiinit)
{
    called_func = PXENV_UNDI_INITIALIZE;
}

pxenv_ret_code_t
pxenv_undi_reset_nic (pxenv_undi_reset_nic_t *undireset)
{
    called_func = PXENV_UNDI_RESET_NIC;
}

pxenv_ret_code_t
pxenv_undi_shutdown (pxenv_undi_shutdown_t *undishut)
{
    called_func = PXENV_UNDI_SHUTDOWN;
}

pxenv_ret_code_t
pxenv_undi_open (pxenv_undi_open_t *undiopen)
{
    called_func = PXENV_UNDI_OPEN;
}

pxenv_ret_code_t
pxenv_undi_close (pxenv_undi_close_t *undiclose)
{
    called_func = PXENV_UNDI_CLOSE;
}

pxenv_ret_code_t
pxenv_undi_transmit (pxenv_undi_transmit_t *unditrans)
{
    called_func = PXENV_UNDI_TRANSMIT;
}

pxenv_ret_code_t
pxenv_undi_set_mcast_addr (pxenv_undi_set_mcast_addr_t *undimcast)
{
    called_func = PXENV_UNDI_SET_MCAST_ADDR;
}

pxenv_ret_code_t
pxenv_undi_set_station_addr (pxenv_undi_set_station_addr_t *undistat)
{
    called_func = PXENV_UNDI_SET_STATION_ADDR;
}

pxenv_ret_code_t
pxenv_undi_set_packet_filter (pxenv_undi_set_packet_filter_t *undifilter)
{
    called_func = PXENV_UNDI_SET_PACKET_FILTER;
}

pxenv_ret_code_t
pxenv_undi_get_information (pxenv_undi_get_information_t *undiinfo)
{
    called_func = PXENV_UNDI_GET_INFORMATION;
}

pxenv_ret_code_t
pxenv_undi_get_statistics (pxenv_undi_get_statistics_t *undistats)
{
    called_func = PXENV_UNDI_GET_STATISTICS;
}

pxenv_ret_code_t
pxenv_undi_clear_statistics (pxenv_undi_clear_statistics_t *undiclrstats)
{
    called_func = PXENV_UNDI_CLEAR_STATISTICS;
}

pxenv_ret_code_t
pxenv_undi_initiate_diags (pxenv_undi_initiate_diags_t *undidiag)
{
    called_func = PXENV_UNDI_INITIATE_DIAGS;
}

pxenv_ret_code_t
pxenv_undi_force_interrupt (pxenv_undi_force_interrupt_t *undiint)
{
    called_func = PXENV_UNDI_FORCE_INTERRUPT;
}

pxenv_ret_code_t
pxenv_undi_get_mcast_addr (pxenv_undi_get_mcast_addr_t *undimcast)
{
    called_func = PXENV_UNDI_GET_MCAST_ADDR;
}

pxenv_ret_code_t
pxenv_undi_get_nic_type (pxenv_undi_get_nic_type_t *undinic)
{
    called_func = PXENV_UNDI_GET_NIC_TYPE;
}

pxenv_ret_code_t
pxenv_undi_get_iface_info (pxenv_undi_get_iface_info_t *undiiface)
{
//    called_func = PXENV_UNDI_GET_IFACE_INFO;
}

pxenv_ret_code_t
pxenv_undi_get_state (pxenv_undi_get_state_t *undistate)
{
    called_func = PXENV_UNDI_GET_STATE;
}

pxenv_ret_code_t
pxenv_undi_isr (pxenv_undi_isr_t *undiisr)
{
    called_func = PXENV_UNDI_ISR;
}

/* Prototypes for the PREBOOT PXE API functions */
pxenv_ret_code_t
pxenv_unload_stack (pxenv_unload_stack_t *stackptr)
{
    called_func = PXENV_UNLOAD_STACK;
}

pxenv_ret_code_t
pxenv_get_cached_info (pxenv_get_cached_info_t *cinfoptr)
{
    called_func = PXENV_GET_BINL_INFO; ;
}

pxenv_ret_code_t
pxenv_restart_tftp (pxenv_restart_tftp_t *session)
{
    called_func = PXENV_RESTART_TFTP;
}

pxenv_ret_code_t
pxenv_start_undi (pxenv_start_undi_t *startundi)
{
    called_func = PXENV_START_UNDI;
}

pxenv_ret_code_t
pxenv_stop_undi (pxenv_stop_undi_t *stopundi)
{
    called_func = PXENV_STOP_UNDI;
}

pxenv_ret_code_t
pxenv_start_base (pxenv_start_base_t *startbase)
{
    called_func = PXENV_START_BASE;
}

pxenv_ret_code_t
pxenv_stop_base (pxenv_stop_base_t *stopbase)
{
    called_func = PXENV_STOP_BASE;
}


